{ nixpkgs, pkgs, kernel, authorized-keys }:

let
  buildSystem = fdt: nixpkgs.lib.nixosSystem {
    system = "aarch64-linux";

    modules = [
      "${nixpkgs}/nixos/modules/installer/sd-card/sd-image-aarch64.nix"
      {
        nixpkgs.overlays = [
          (final: super: {
            zfs = super.zfs.overrideAttrs (_: {
              meta.platforms = [ ];
            });
          })
        ];

        users.users.root.initialHashedPassword = "";
        users.users.root.openssh.authorizedKeys.keys = authorized-keys;

        services.openssh.enable = true;

        networking.hostName = "nixos-aarch64";

        boot.kernelPackages = kernel;
        boot.kernelParams = [ "console=ttyS2,1500000n8" ];

        hardware.deviceTree.name = fdt;

        system.stateVersion = "23.05";
      }
    ];
  };
in
{
  quartz64a = buildSystem "rockchip/rk3566-quartz64-a.dtb";
  quartz64b = buildSystem "rockchip/rk3566-quartz64-b.dtb";
  soquartz-model-a = buildSystem "rockchip/rk3566-soquartz-model-a.dtb";
  soquartz-cm4 = buildSystem "rockchip/rk3566-soquartz-cm4.dtb";
  soquartz-blade = buildSystem "rockchip/rk3566-soquartz-blade.dtb";
  rock64 = buildSystem "rockchip/rk3328-rock64.dtb";
  rockPro64 = buildSystem "rockchip/rk3399-rockpro64.dtb";
  rockPro64v2 = buildSystem "rockchip/rk3399-rockpro64-v2.dtb";
  roc-pc-rk3399 = buildSystem "rockchip/rk3399-roc-pc.dtb";
  pinebookPro = buildSystem "rockchipo/rk3399-pinebook-pro.dtb";
}
